import {
  ChangeDetectionStrategy,
  Component,
  Input,
  Output,
  ViewEncapsulation,
} from "@angular/core";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {
  LanguagesEnum,
  LanguagesEnumHelper,
} from "@app/shared/enums/languages.enum";
import {
  BehaviorSubject,
  Observable,
} from "rxjs";
import {ObservableUtil} from "solidify-frontend";

@Component({
  selector: "dlcm-language-selector",
  templateUrl: "./language-selector.presentational.html",
  styleUrls: ["./language-selector.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
  encapsulation: ViewEncapsulation.None,
})
export class LanguageSelectorPresentational extends SharedAbstractPresentational {
  languageList: LanguagesEnum[] = [LanguagesEnum.en, LanguagesEnum.fr, LanguagesEnum.de];

  private readonly _languageBS: BehaviorSubject<LanguagesEnum | undefined> = new BehaviorSubject<LanguagesEnum | undefined>(undefined);

  @Input()
  currentLanguage: LanguagesEnum;

  @Output("languageChange")
  readonly languageObs: Observable<LanguagesEnum | undefined> = ObservableUtil.asObservable(this._languageBS);

  private get _language(): LanguagesEnum | undefined {
    return this._languageBS.getValue();
  }

  private set _language(value: LanguagesEnum | undefined) {
    if (this._languageBS.value !== value) {
      this._languageBS.next(value);
    }
  }

  get language(): LanguagesEnum | undefined {
    return this._language;
  }

  @Input()
  set language(language: LanguagesEnum | undefined) {
    this._language = language;
  }

  get languagesEnumHelper(): typeof LanguagesEnumHelper {
    return LanguagesEnumHelper;
  }

  useLanguage(language: LanguagesEnum): void {
    this._languageBS.next(language);
  }

  getPathToImage(language: LanguagesEnum): string {
    return "assets/language-icons/" + language + ".svg";
  }
}
